;;; Some values ;;;
TRUE		=	-1	; Logical True value
FALSE		=	 0	; Logical False value

BYTEBITS	=	 8	; bits in a byte (smallest addressable value)

CHARABITS	=	 0	; bits of char in address
CELLABITS	=	 2	; bits of cell in address
PAGEABITS	=	12	; bits of page in address


;;; Deduced values ;;;
CHARSIZE	=	(1 SHL CHARABITS)	; size of a char in bytes
CELLSIZE	=	(1 SHL CELLABITS)	; size of a cell in bytes
PAGESIZE	=	(1 SHL PAGEABITS)	; size of a page in bytes
CHARMASK	=	NOT (CHARSIZE - 1)	; mask of chars in address
CELLMASK	=	NOT (CELLSIZE - 1)	; mask of cells in address
PAGEMASK	=	NOT (PAGESIZE - 1)	; mask of pages in address
BITSPERCHAR	=	CHARSIZE*BYTEBITS	; bits in a char
BITSPERCELL	=	CELLSIZE*BYTEBITS	; bits in a cell



;;; Defining FORTH words ;;;

Macro	DeclName	name
	local	len
  Segment NameSpace
    NAMEADDR = $
	db	len
	db	<name>
    len = $ - NAMEADDR - 1
  EndS	NameSpace
EndM	DeclName

Macro	Define		name
	local OBJADDR
	OBJADDR = $
	DeclName	<name>
  Segment NameTable
	dd	NAMEADDR, OBJADDR
  EndS	  NameTable
EndM	Define
Macro	Definline	name	; inline function 
EndM	Definline


;;; Absolute jump macros ;;;
Macro AbsJmp dest,tmpreg
	ifb <tmpreg>
	  push (offset dest)		; 5,  2  , 1
	  retn				; 1, 10+m, 5
	elif <>
	  push (offset dest)
	  retn
	else
	  mov tmpreg,(offset dest)	; 5,  2  , 1
	  jmp tmpreg			; 2,  7+m, 5
	endif
EndM  AbsJmp
Macro AbsLngJmp dest
	push (seg dest)			; 5
	push (offset dest)		; 5
	retf				; 1
EndM  AbsLngJmp
Macro	ACall dest,align?
	ifnb <align?>
	  AlignP 4,2
	endif
	ds:
	mov eax,offset dest
	call eax
EndM	ACall


;;; Aerial (extra-light) threads.
Struc AerialThread	_bx,_si,_di,_bp,_sp,eesp,resp ; Lighter than Light !
  AT_SAVE_EBX		dd	?	; initial Top of Stack
  AT_SAVE_ESI		dd	?	; Data Stack Pointer
  AT_SAVE_EDI		dd	?	; Return Stack Pointer
  AT_SAVE_EBP		dd	?	; Those two are reserved for the
  AT_SAVE_ESP		dd	?	;; threading mechanism.
  AT_SAVE_RESP		dd	?	; ESP save when going ret-threaded
  AT_STATE		dd	?	; flags used for thread sync'
  AT_MAX_ESI		dd	?	; bottom of data stack
  AT_MAX_EDI		dd	?	; bottom of return stack
  AT_LINKBACK		dd	?	; link back to thread object
EndS  AerialThread




;;; Virtual arrays

; parameter is the address of a character*attribute virtual array.

Struc VirtualArray
param	dd	?	; parameter		( va )
width	dd	?	; get number of columns	( va -- n )
height	dd	?	; get number of lines	( va -- n )
get	dd	?	; get char & attribute  ( va x y -- ch attr )
put	dd	?	; set char & attribute  ( va x y ch attr -- )
EndS  VirtualArray

